import os
import subprocess
import sys


# general use
def get_output(*args, **kwargs):
    res = subprocess.check_output(*args, shell=True, **kwargs)
    decoded = res.decode("utf-8")
    return decoded.strip()


# python-gssapi related
def import_gssapi_extension(name):
    """Import a GSSAPI extension module

    This method imports a GSSAPI extension module based
    on the name of the extension (not including the
    'ext_' prefix).  If the extension is not available,
    the method retuns None.

    Args:
        name (str): the name of the extension

    Returns:
        module: Either the extension module or None
    """

    try:
        path = "gssapi.raw.ext_{0}".format(name)
        __import__(path)
        return sys.modules[path]
    except ImportError:
        return None


# krb5-plugin related
_PLUGIN_DIR = None


def find_plugin_dir():
    global _PLUGIN_DIR
    if _PLUGIN_DIR is not None:
        return _PLUGIN_DIR

    # if we've set a LD_LIBRARY_PATH, use that first
    ld_path_raw = os.environ.get("LD_LIBRARY_PATH")
    if ld_path_raw is not None:
        # first, try assuming it's just a normal install

        ld_paths = [path for path in ld_path_raw.split(":") if path]

        for ld_path in ld_paths:
            if not os.path.exists(ld_path):
                continue

            _PLUGIN_DIR = _decide_plugin_dir(_find_plugin_dirs_installed(ld_path))
            if _PLUGIN_DIR is None:
                _PLUGIN_DIR = _decide_plugin_dir(_find_plugin_dirs_src(ld_path))

            if _PLUGIN_DIR is not None:
                break

    # if there was no LD_LIBRARY_PATH, or the above failed
    if _PLUGIN_DIR is None:
        lib_dir = os.path.join(get_output("krb5-config --prefix"), "lib64")
        _PLUGIN_DIR = _decide_plugin_dir(_find_plugin_dirs_installed(lib_dir))

    # /usr/lib64 seems only to be distinct on Fedora/RHEL/Centos family
    if _PLUGIN_DIR is None:
        lib_dir = os.path.join(get_output("krb5-config --prefix"), "lib")
        _PLUGIN_DIR = _decide_plugin_dir(_find_plugin_dirs_installed(lib_dir))

    if _PLUGIN_DIR is not None:
        _PLUGIN_DIR = os.path.normpath(_PLUGIN_DIR)
        return _PLUGIN_DIR
    else:
        return None


def _decide_plugin_dir(dirs):
    if dirs is None:
        return None

    # the shortest path is probably more correct
    shortest_first = sorted(dirs, key=len)

    for path in shortest_first:
        # check to see if it actually contains .so files
        if get_output('find %s -name "*.so"' % path):
            return path

    return None


def _find_plugin_dirs_installed(search_path):
    try:
        options_raw = get_output(
            "find %s/ -type d \( ! -executable -o ! -readable \) "
            "-prune -o "
            '-type d -path "*/krb5/plugins" -print' % search_path,
            stderr=subprocess.STDOUT,
        )
    except subprocess.CalledProcessError:
        options_raw = None

    if options_raw:
        return options_raw.split("\n")
    else:
        return None


def _find_plugin_dirs_src(search_path):
    options_raw = get_output("find %s/../ -type d -name plugins" % search_path)

    if options_raw:
        return options_raw.split("\n")
    else:
        return None



import os
import unittest

from k5test import _utils, realm


# test case class
class KerberosTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.realm = realm.K5Realm()

    @classmethod
    def tearDownClass(cls):
        cls.realm.stop()
        del cls.realm


# decorators
def gssapi_extension_test(extension_name, extension_text):
    def make_ext_test(func):
        def ext_test(self, *args, **kwargs):
            if _utils.import_gssapi_extension(extension_name) is None:
                self.skipTest(
                    "The %s GSSAPI extension is not supported by "
                    "your GSSAPI implementation" % extension_text
                )
            else:
                func(self, *args, **kwargs)

        return ext_test

    return make_ext_test


_KRB_VERSION = None


def krb_minversion_test(target_version, problem, provider=None):
    global _KRB_VERSION
    if _KRB_VERSION is None:
        _KRB_VERSION = _utils.get_output("krb5-config --version")
        _KRB_VERSION = _KRB_VERSION.split(" ")[-1].split(".")

    def make_ext_test(func):
        def ext_test(self, *args, **kwargs):
            if _KRB_VERSION < target_version.split(".") and (
                not provider or self.realm.lower() == provider.lower()
            ):
                self.skipTest(
                    "Your GSSAPI (version %s) is known to have "
                    "problems with %s" % (_KRB_VERSION, problem)
                )
            else:
                func(self, *args, **kwargs)

        return ext_test

    return make_ext_test


def krb_plugin_test(plugin_type, plugin_name):
    # TODO(directxman12): add a way to make this look for
    # platform-specific library extensions
    krb5_plugin_path = _utils.find_plugin_dir()
    plugin_path = None
    if krb5_plugin_path:
        plugin_path = os.path.join(krb5_plugin_path, plugin_type, f"{plugin_name}.so")

    def make_krb_plugin_test(func):
        def krb_plugin_test(self, *args, **kwargs):
            if not plugin_path or not os.path.exists(plugin_path):
                self.skipTest(
                    "You do not have the GSSAPI {type}"
                    "plugin {name} installed".format(type=plugin_type, name=plugin_name)
                )
            else:
                func(self, *args, **kwargs)

        return krb_plugin_test

    return make_krb_plugin_test


def krb_provider_test(providers, problem):
    def make_krb_provider_test(func):
        def krb_provider_test(self, *args, **kwargs):
            provider_list = [p.lower() for p in providers]
            if self.realm.provider.lower() not in provider_list:
                self.skipTest(
                    f"Your GSSAPI (provider {self.realm.provider}) "
                    f"is known to have problems with {problem}"
                )
            else:
                func(self, *args, **kwargs)

        return krb_provider_test

    return make_krb_provider_test
